// メモリ節約型設定ファイル読み込みライブラリ (diminutive configuration file loader)

/*!
 * @file dimiConfigLoader.h
 * @brief メモリ節約型ファイル読み込みライブラリのインクルード
 */

#ifndef DIMI_CONFIG_LOADER_H
#define DIMI_CONFIG_LOADER_H

#define DIMICONF_OK           0  /**< 正常 */
#define DIMICONF_NONE         1  /**< 指定したセクションまたはキーが存在しない */
#define DIMICONF_E_PARAM    (-1) /**< 引数誤り */
#define DIMICONF_E_ASSIGN   (-2) /**< セクション未設定 */
#define DIMICONF_E_MEMORY   (-3) /**< メモリ不足 */
#define DIMICONF_E_OPEN     (-4) /**< 設定ファイルオープンエラー */
#define DIMICONF_E_OVERFLOW (-5) /**< 設定ファイルの設定項目が多い */
#define DIMICONF_E_SYNTAX   (-6) /**< 設定ファイルの構文誤り */

#ifdef __cplusplus
extern "C" {
#endif

/*!
 * @brief ファイル読み込みライブラリのオープン
 *
 * @param[out] conf ファイル読み込みライブラリのハンドルの返却先
 * @param[in]  fname 設定ファイル名
 * @retval DIMICONF_OK 成功
 * @retval DIMICONE_E_PARAM 引数誤り
 * @retval DIMICONF_E_MEMORY メモリ不足
 * @retval DIMICONF_E_OPEN 設定ファイルのオープンに失敗
 * @retval DIMICONF_E_OVERFLOW 設定項目超過
 * @retval DIMICONF_E_SYNTAX 設定ファイルの構文誤り
 */
int DimiConf_Open(void **conf, const char *fname);

/*!
 * @brief ファイル読み込みライブラリのクローズ
 *
 * @param[in] conf ファイル読み込みライブラリのハンドル
 * @retval DIMICONF_OK 成功
 */
int DimiConf_Close(void *conf);

/*!
 * @brief セクション名、キー名に関連付けられた文字列の取得
 *
 * @param[in] conf ファイル読み込みライブラリのハンドル
 * @param[in] section_name セクション名
 * @param[in] key_name キー名
 * @param[in] def デフォルト値
 * @return セクション名、キー名に関連付けられた文字列へのポインタ。
 * キーが存在しない場合は、引数defの値を返却する。
 * 引数に誤りがある場合には、NULLを返却する。
 */
char * DimiConf_GetString(void *conf, const char *section_name, const char *key_name, char *def);

/*!
 * @brief セクション名、キー名に関連付けられた文字列の出現順位指定付での取得
 * 出現順位に指定した値+1番目に登録されているセクション名、キー名に
 * 関連付けられている文字列を取得する。
 * 出現順位に1を指定すれば指定したセクション名、キー名で2番目に登録されている
 * 文字列を取得することになる。
 *
 * @param[in] conf ファイル読み込みライブラリのハンドル
 * @param[in] section_name セクション名
 * @param[in] key_name キー名
 * @param[in] pos 出現順位
 * @param[in] def デフォルト値
 * @return セクション名、キー名に関連付けられた文字列へのポインタ。
 * キーが存在しない場合は、引数defの値を返却する。
 * 引数に誤りがある場合には、NULLを返却する。
 */
char * DimiConf_GetSiblingString(void *conf, const char *section_name, const char *key_name, int pos, char *def);

#ifdef __cplusplus
}
#endif

#endif // DIMI_CONFIG_LOADER_H

